@@ -35,7 +35,9 @@ class HippyBot(JabberBot):
 
     _global_commands = []
     _command_aliases = {}
+    _all_msg_handlers = []
     _last_message = ''
+    _last_send_time = time.time()
     _restart = False
 
     def __init__(self, config):
@@ -97,6 +99,13 @@ class HippyBot(JabberBot):
             to = False
         return to, mess
 
+    def send_message(self, mess):
+        """Send an XMPP message
+        Overridden from jabberbot to update _last_send_time
+        """
+        self._last_send_time = time.time()
+        self.connect().send(mess)
+
     def callback_message(self, conn, mess):
         """Message handler, this is where we route messages and transform
         direct messages and message aliases into the command that will be
@@ -108,9 +117,25 @@ class HippyBot(JabberBot):
 
         at_msg, message = self.to_bot(message)
 
-        cmd = message.split(' ')[0]
+        if len(self._all_msg_handlers) > 0:
+            for handler in self._all_msg_handlers:
+                try:
+                    handler(mess)
+                except Exception, e:
+                    self.log.exception(
+                            'An error happened while processing '
+                            'a message ("%s") from %s: %s"' %
+                            (mess.getType(), mess.getFrom(),
+                                traceback.format_exc(e)))
+
+        if u' ' in message:
+            cmd = message.split(u' ')[0]
+        else:
+            cmd = ''
+
         if cmd in self._command_aliases:
-            message = u"%s%s" % (self._command_aliases[cmd], message[len(cmd):])
+            message = u"%s%s" % (self._command_aliases[cmd],
+                                message[len(cmd):])
             cmd = self._command_aliases[cmd]
 
         ret = None
@@ -148,8 +173,8 @@ class HippyBot(JabberBot):
         to HipChat, as XMPP ping doesn't seem to cut it.
         """
         if self.PING_FREQUENCY \
-            and time.time() - self._JabberBot__lastping > self.PING_FREQUENCY:
-            self._JabberBot__lastping = time.time()
+            and time.time() - self._last_send_time > self.PING_FREQUENCY:
+            self._last_send_time = time.time()
             self.send_message(' ')
 
     @botcmd
@@ -191,10 +216,15 @@ class HippyBot(JabberBot):
                 # hippybot, e.g. they can just be said in the channel
                 self._global_commands.extend(getattr(plugin,
                                                 'global_commands', []))
-                # Check for "special commands", e.g. those that can be
+                # Check for "special commands", e.g. those that can't be
                 # represented in a python method name
                 self._command_aliases.update(getattr(plugin,
                                                 'command_aliases', {}))
+
+                # Check for handlers for all XMPP message types,
+                # this can be used for low-level checking of XMPP messages
+                self._all_msg_handlers.extend(getattr(plugin,
+                                                'all_msg_handlers', []))
             else:
                 funcs = [(name, command)]
 
